
import socket
from struct import unpack_from
import numpy as np

from srsgui.inst.component import Component
from srsgui.inst.commands import Command, GetCommand,\
                                 BoolCommand, BoolGetCommand,\
                                 IntCommand, IntGetCommand, IntSetCommand,\
                                 FloatCommand, FloatSetCommand, FloatGetCommand

from srsgui.inst.indexcommands import IndexCommand, IndexGetCommand, \
                                      IntIndexCommand, IntIndexGetCommand, \
                                      BoolIndexCommand, BoolIndexGetCommand,\
                                      FloatIndexCommand


class Stream(Component):
    ChannelDict = {
        'X':  0,
        'XY': 1,
        'RT': 2,
        'XYRT': 3
    }

    FormatDict = {
        'float32': 0,
        'int16': 1
    }

    PacketSizeDict = {
        1024: 0,
        512:  1,
        256:  2,
        128:  3
    }

    OptionDict = {
        'little_endian': 1,
        'data_integrity_checking': 2
    }

    channel = IntCommand('STREAMCH')
    max_rate = FloatGetCommand('STREAMRATEMAX')
    rate = IntCommand('STREAMRATE')
    format = IntCommand('STREAMFMT')
    packet_size = IntCommand('STREAMPCKT')
    port = IntCommand('STREAMPORT')
    option = IntCommand('STREAMOPTION')
    enable = BoolCommand('STREAM')

    def _prepare(self):
        self.udp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.udp_socket.bind(('', self.port))
        self.timeout = 10
        self.udp_socket.settimeout(self.timeout)
        self.prepared_channel = self.channel

        self.prepared_packet_size = int(list(Stream.PacketSizeDict.keys())[self.packet_size])
        self.unpack_format = '>{}h'.format(self.prepared_packet_size // 2) if self.format else \
                             '>{}f'.format(self.prepared_packet_size // 4)

    def receive_packet(self):
        buffer, _ = self.udp_socket.recvfrom(self.prepared_packet_size + 4)
        packet_number = unpack_from('>I', buffer)[0] & 0xffff
        vals = unpack_from(self.unpack_format, buffer, 4)

        arr = None
        if self.prepared_channel == 0:
            arr = np.array(vals)

        elif self.prepared_channel == 1:
            rows = len(vals) // 2
            mat = np.transpose(np.reshape(vals, (rows, 2)))
            r = np.sqrt(np.square(mat[0]) + np.square(mat[1]))
            th = 180.0 / np.pi * np.arctan2(mat[1], mat[0])
            arr = np.append(np.append(mat, np.reshape(r, (1, rows)), axis=0),
                            np.reshape(th, (1,rows)), axis=0)

        elif self.prepared_channel == 2:
            rows = len(vals) // 2
            mat = np.transpose(np.reshape(vals, (rows, 2)))
            angle = np.pi / 180.0 * mat[1]
            x = mat[0] * np.cos(angle)
            y = mat[0] * np.sin(angle)
            arr = np.append(np.append(np.reshape(x, (1, rows)),
                                      np.reshape(y, (1, rows)), axis=0),
                            mat, axis=0)

        elif self.prepared_channel == 3:
            row = len(vals) // 4
            arr = np.transpose(np.reshape(vals, (row, 4)))

        return arr, packet_number

    def start(self):
        self._prepare()
        self.enable = True

    def stop(self):
        self.enable = False
        self.udp_socket.close()


class Auto(Component):
    def set_phase(self):
        self.comm.send('APHS')

    def set_range(self):
        self.comm.send('ARNG')

    def set_scale(self):
        self.comm.send('ASCL')




import time
import logging
import socket
import struct
import threading

from srsgui import Task
from srsgui import IntegerInput, ListInput
from instruments.sr865.sr865 import SR865
from instruments import get_sr865


class StreamingTask(Task):
    Duration = 'duration'
    Channels = 'Channels'
    DataFormat = 'data format'
    PacketSize = 'packet size'
    Port = 'udp port'

    input_parameters = {
        Duration: IntegerInput(10, ' s', 1, 3600, 1),
        Channels: ListInput(['X', 'X, Y', 'R, Theta', 'X, Y, R, Theta'], 1),
        DataFormat: ListInput(['Float32', 'Int16']),
        PacketSize: ListInput(['1024', '512', '256', '128']),
        Port: IntegerInput(1865, '', 1024, 65535, 1)
    }

    def setup(self):
        self.logger = logging.getLogger(__file__)
        self.lia = get_sr865(self)
        
        print(self.lia.query_text('*idn?'))

        self.params = self.get_all_input_parameters()


        self.lia.stream.channel = self.params[self.Channels]
        self.lia.stream.format = self.params[self.DataFormat]
        self.lia.stream.packet_size = self.params[self.PacketSize]
        self.lia.stream.port = self.params[self.Port]


        self.logger.info('Channels: {}, Data format: {}, Packet_size: {}, Port: {}'
                         .format(list(self.lia.stream.ChannelDict.keys())[self.lia.stream.channel],
                                 list(self.lia.stream.FormatDict.keys())[self.lia.stream.format],
                                 list(self.lia.stream.PacketSizeDict.keys())[self.lia.stream.packet_size],
                                 self.lia.stream.port,
                                 )
                         )

        # Mark the time 0
        self.init_time = time.time()

    def test(self):
        while time.time() - self.init_time < self.params[self.Duration]:
            if not self.is_running():
                break

            start_time = time.time()
            for i in range(100):
                print(self.lia.query_text('*idn?'))
            finish_time = time.time()
            print('Time for 100 *idn: {}'.format(finish_time - start_time))
            break

    def cleanup(self):
        pass





from .instruments.sr865.vxi11interface import Vxi11Interface
from .instruments.sr865.visainterface import VisaInterface

from .instruments.sr865.sr865 import SR865
from .instruments.get_instruments import get_sr865

__version__ = "0.0.3"


from .vxi11interface import Vxi11Interface
from .visainterface import VisaInterface

from srsgui import Instrument
from srsgui.inst import TcpipInterface, SerialInterface
from srsgui.task.inputs import IntegerListInput, BoolInput, \
                               Ip4Input, IntegerInput, FindListInput

from .components import Stream


class SR865(Instrument):
    _IdString = 'SR865'

    available_interfaces = [
        [
            Vxi11Interface,
            {
                'ip_address': Ip4Input('192.168.1.10'),
            }
        ],

        [
            VisaInterface,
            {
                'resource': FindListInput(),
            }
        ],

        [
            TcpipInterface,
            {
                'ip_address': Ip4Input('192.168.1.10'),
                'port': 23
            }
        ],

        [
            SerialInterface,
            {
                'port': FindListInput(),
                'baud_rate': IntegerListInput([9600, 19200, 38400, 115200,
                                               230400, 460800], 3)
            }
        ],
    ]

    def __init__(self, interface_type=None, *args):
        super().__init__(interface_type, *args)

        self.stream = Stream(self)

    def connect(self, interface_type, *args):
        super().connect(interface_type, *args)
        if isinstance(self.comm, TcpipInterface):
            print(self.query_text(''))  # Read out the initial string



import os
from setuptools import setup, find_packages, find_namespace_packages


def find_version(*args):
    """ Find version string starting with __version__ in a file"""

    f_path = os.path.join(os.path.dirname(__file__), *args)
    with open(f_path) as f:
        for line in f:
            if line.startswith('__version__'):
                break
    version_strings = line.split('"')
    if len(version_strings) != 3:
        raise ValueError('Version string is not enclosed inside " ".')
    return version_strings[1]


description = open('README.md').read()
version_string = find_version('srsinst/sr860', '__init__.py')

setup(
    name='srsinst.sr860',
    version=version_string,
    description='Communication package for SRS Lock-In Amplifier SR860 series',
    packages=['srsinst.sr860', 'srsinst.sr860.instruments', 'srsinst.sr860.instruments.sr865',
              'srsinst.sr860.tasks', 'srsinst.sr860.plots'],
             # find_namespace_packages(),
    package_data={
        'srsinst.sr860': ['*.taskconfig']
    },
    # include_package_data=True,
    long_description=description,
    long_description_content_type='text/markdown',
    python_requires='>=3.7',
    install_requires=[
        "numpy",
        "python-vxi11",
        "srsgui>=0.1.7"
    ],
    extras_require={
        'gui': ['matplotlib', 'pyside2']
    },
    entry_points={
        'console_scripts': [
            'sr860 = srsinst.sr860.__main__:main'
        ],
    },

    license="MIT license",
    keywords=["SR860", "SR865A", "SRS", "Stanford Research Systems"],
    author="Chulhoon Kim",
    # author_email="chulhoonk@yahoo.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Scientific/Engineering :: Physics"
    ]
)
